import React, { ChangeEvent, useCallback, useState } from 'react';
import { InputBinder } from '../types/inputBinder';

export const useObjectState = <T extends object>(initial: T) => {
  const [state, setState] = useState<T>(initial || {} as T);

  const bindProperty = useCallback(<K extends keyof T>(property: K) => ({
    value: state[property] ?? '',
    name: property,
    onChange: (event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement | HTMLSelectElement>) =>
      setState(value => (
        {
          ...value,
          [event.target.name]: (
            (typeof value[property] === 'number') ?
              Number(event.target.value) || 0 :
              event.target.value)
        }
      ))
  }), [state])

  const bindPropertyCheck = useCallback(<K extends keyof T>(property: K) => ({
    checked: !!state[property],
    name: property,
    onChange: (event: ChangeEvent<HTMLInputElement>) => setState(value => ({
      ...value, [event.target.name]: (event.target.checked)
    })),
    readOnly: true
  }), [state])

  const update = useCallback((updates: Partial<T>) => setState(s => ({ ...s, ...updates })), [])

  const reset = useCallback(() => {
    setState(initial);
  }, [initial])

  return {
    bindProperty,
    bindPropertyCheck,
    update,
    state,
    setState,
    reset
  }
}

export const useObjectStateWrapper = <T extends object>(state: T, setState: React.Dispatch<React.SetStateAction<T>>) => {

  const bindProperty = useCallback(<K extends keyof T>(property: K): InputBinder => ({
    value: state[property] ?? '',
    name: property.toString(),
    onChange: (event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement | HTMLSelectElement>) =>
      setState(value => (
        {
          ...value,
          [event.target.name]: (
            (typeof value[property] === 'number') ?
              Number(event.target.value) || 0 :
              event.target.value)
        }
      ))
  }), [setState, state])

  const bindPropertyCheck = useCallback(<K extends keyof T>(property: K) => ({
    checked: !!state[property],
    name: property,
    onChange: (event: ChangeEvent<HTMLInputElement>) => setState(value => ({
      ...value, [event.target.name]: (event.target.checked)
    })),
    readOnly: true
  }), [setState, state])

  const update = useCallback((updates: Partial<T> | ((arg: T) => Partial<T>)) =>
    setState(s => ({ ...s, ...(typeof updates === 'function' ? updates(s) : updates) }
    )), [setState])

  return {
    bindProperty,
    bindPropertyCheck,
    update,
    state,
    setState
  }
}